<?php

namespace Drupal\mcapi\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;

/**
 * Defines the transaction workflow entity class, i.e. the workflow definition.
 *
 * @ConfigEntityType(
 *   id = "mc_workflow",
 *   label = @Translation("Transaction workflow"),
 *   config_prefix = "mc_workflow",
 *   admin_permission = "configure mcapi",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *   },
 *   handlers = {
 *     "access" = "Drupal\mcapi\Entity\Access\WorkflowAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\mcapi\Form\WorkflowEdit",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider",
 *     },
 *     "list_builder" = "Drupal\mcapi\Entity\ListBuilder\WorkflowListBuilder"
 *   },
 *   links = {
 *     "add-form" = "/admin/accounting/workflow/add",
 *     "edit-form" = "/admin/accounting/workflow/{mc_workflow}/edit",
 *     "delete-form" = "/admin/accounting/workflow/{mc_workflow}/delete",
 *     "collection" = "/admin/accounting/workflows"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "summary",
 *     "home",
 *     "creation",
 *     "direction",
 *     "states",
 *     "disabled"
 *   }
 * )
 */
class Workflow extends ConfigEntityBase {

  const CREDIT = 'credit';
  const BILL = 'bill';
  const THIRDPARTY = '3rdparty';

  /**
   * Unique ID
   * @var string
   */
  public string $id;

  /**
   * Name to show in lists
   * @var string
   */
  public string $label;

  /**
   * Short description
   * @var string
   */
  public string $summary = '';

  /**
   * Whether this workflow originates locally
   * @var string
   */
  public $home;

  /**
   * Whether this is BILL or CREDIT or THIRDPARTY
   * @var string
   */
  public string $direction = self::THIRDPARTY;

  /**
   * Creation parameters.
   * @var stdClass with properties string state, and bool confirm
   */
  public $creation;

  /**
   * Structured data
   * @var array
   */
  public array $states = [];

  /**
   * TRUE if the workflow is disabled.
   * @var bool
   */
  public bool $disabled = FALSE;

  public function __construct(array $values, $entity_type) {
    if (empty($values['home'])) {
      // localHomename is not set during installation.
      $values['home'] = self::localHomeName();
    }
    return parent::__construct($values, $entity_type);
  }

  // This is just to help with upgrades.
  private function localHomeName() : string {
    if ($site_name = \Drupal::config('system.site')->get('name')) {
      $site_name = str_replace(' ', '', $site_name);
      return preg_replace('/[^a-z_]/', '', strtolower($site_name));
    }
    return '';
  }


  /**
   * @param type $include_disabled
   * @return type
   */
  public static function loadAlphabetical($include_disabled = FALSE) {
    $all = static::loadMultiple();
    usort(
      $all,
      function ($a, $b){return $a->label <=> $b->label;}
    );
    // TEMP
    foreach ($all as $wf) {
      if (!isset($wf->home)){// temp
        $wf->home = \Drupal::config('mcapi_cc.settings')->get('node_name');
      }
    }
    // sort by whether or not the workflow is local
    $home = \Drupal::config('mcapi_cc.settings')->get('node_name');
    usort(
      $all,
      function ($a, $b) use ($home) {return ($a->home == $home) <=> ($b->home == $home);}
    );
    if ($include_disabled) {
      uasort($all, function ($a, $b) {$a->disabled < $b->disabled;});
    }
    else {
      $all = array_filter($all, function ($w) {return $w->disabled <> TRUE;});
    }
    return $all;
  }

  function isLocal() : bool {
    static $home;
    if (!isset($home)) {
      $home = $this->localHomeName();
    };
    return $home == $this->home;
  }


}
