<?php

namespace Drupal\mcapi\EventSubscriber;

use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionSaveEvent;
use Drupal\mcapi\Entity\Transaction;
use Drupal\user\Entity\User;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Mail\MailManager;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Core\Logger\LoggerChannel;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use CreditCommons\TransactionInterface as CCTransactionInterface;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;


/**
 * Subscriber showing the two Mcapi events.
 */
class TransactionSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * @var Drupal\Core\Logger\LoggerChannel
   */
  private LoggerChannel $logger;

  /**
   * @var int
   */
  private int $currentUserId;

  /**
   * @var MailManager
   */
  private MailManager $mailManager;

  /**
   * mcapi.settings
   * @var Config
   */
  protected $signatureMail;
  protected $notificationMail;

  /**
   * Constructor
   *
   * @param ConfigFactoryInterface $config_factory
   * @param AccountInterface $account
   * @param MailManager $mail_manager
   * @param LoggerChannelFactory $logger_factory
   */
  function __construct(ConfigFactoryInterface $config_factory, AccountInterface $account, MailManager $mail_manager, LoggerChannelFactory $logger_factory) {
    $config = $config_factory->get('mcapi.settings');
    $this->signatureMail = $config->get('signature_notification');
    $this->notificationMail = $config->get('mail_notification');
    $this->currentUserId = $account->id();
    $this->mailManager = $mail_manager;
    $this->logger = $logger_factory->get('mcapi');
  }

  /**
   * {@inheritDoc}
   * @note there is another transaction event, McapiEvents::ASSEMBLE
   */
  public static function getSubscribedEvents() : array {
    return [
      McapiEvents::ACTION => ['notifyTransactees'],
    ];
  }

  /**
   * Notify transactees when a transaction changes state.
   *
   * @param TransactionSaveEvent $event
   *   $event->getSubject() gives the transaction.
   *   $event->getArguments() yields 'old_state'
   * @param string $eventName
   *   The machine name of the event.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   *
   * @todo In drupal 10.3, add the 3rd typehint.
   */
  public function notifyTransactees(TransactionSaveEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    $transaction = $event->getSubject();
    $args = $event->getArguments();
    $payee_user = $transaction->payee->getOwner();
    $payer_user = $transaction->payer->getOwner();
    if ($payee_user->id() <> $this->currentUserId) {
      $this->notify($payee_user, $transaction);
    }
    if ($payer_user->id() <> $this->currentUserId) {
      $this->notify($payer_user, $transaction);
    }
  }

  /**
   * Generate a basic transaction notification.
   * @param User $participant
   * @param Transaction $transaction
   */
  private function notify(User $participant, Transaction $transaction) {
    if (\Drupal::State()->get('mcapi_mail_stop')) {
      return;
    }
    if ($transaction->state->value == CCTransactionInterface::STATE_PENDING) {
      if (!$transaction->signatures->hasSigned($participant->id())) {
        if ($this->signatureMail['subject'] and $this->signatureMail['body'] ) {
          $message = $this->mailManager->mail(
            'mcapi',
            'signhere',
            $participant->getEmail(),
            $participant->getPreferredLangcode(),
            ['user' => $participant, 'xaction' => $transaction]
          );
          if ($message['result']) {
            $this->logger->notice(
              'Mailed %name to sign transaction @link',
              ['%name' => $participant->getDisplayName(), '@link' => $transaction->toLink()->toString()]
            );
          }
        }
      }
    }
    elseif (!empty($this->notificationMail['subject']) and !empty($this->notificationMail['body'])) {
      $this->mailManager->mail(
        'mcapi',
        'transaction_notification',
        $participant->getEmail(),
        $participant->getPreferredLangcode(),
        [
          'user' => $participant,
          'xaction' => $transaction
        ]
      );
    }
  }

}
